// TinyPIM (c) 1999 Pablo Halpern. File DateBookMenu.h

#ifndef DateBookMenu_dot_h
#define DateBookMenu_dot_h 1

#include <memory>
#include "Menu.h"
#include "AppointmentDisplayList.h"

// Forward reference
class DateBookMenuCatalog;

class DateBookMenu : public Menu
{
public:
  DateBookMenu(DateBook& dateBook, DateBookMenuCatalog* catalog)
    : dateBook_(dateBook), catalog_(catalog), 
      currDate_(DateTime::now()) { }

  void setDate(DateTime dt);

protected:
  void createEntry();
  void showDay();
  void showWeek();
  void showMonth();
  void search();
  void gotoDate();

  virtual void reset();

  DateBook&               dateBook_;
  DateTime                currDate_;
  DateBookMenuCatalog*    catalog_;
};

// Derived class for Monthly view
class MonthlyDateBookMenu : public DateBookMenu
{
public:
  MonthlyDateBookMenu(DateBook& dateBook, 
                      DateBookMenuCatalog* catalog)
    : DateBookMenu(dateBook, catalog), cacheGood_(false) { }

  void mainLoop();

private:
  void displayMonth();
  void showNext();
  void showPrevious();

  virtual void reset();

  bool scoreBoard_[32]; // Index zero unused
  bool cacheGood_;      // true if scoreBoard_ has valid data
};

// Based class for all list-based date book views
class ListBasedDateBookMenu : public DateBookMenu
{
public:
  ListBasedDateBookMenu(DateBook& dateBook, 
                        DateBookMenuCatalog* catalog)
    : DateBookMenu(dateBook, catalog), displayList_(dateBook) { }

protected:
  void viewEntry();
  void editEntry();
  void deleteEntry();

  virtual void reset();

  AppointmentDisplayList  displayList_;
};

// Derived class for Weekly view
class WeeklyDateBookMenu : public ListBasedDateBookMenu
{
public:
  WeeklyDateBookMenu(DateBook& dateBook,
                     DateBookMenuCatalog* catalog);

  void mainLoop();

private:
  void showNext();
  void showPrevious();
};

// Derived class for Daily view
class DailyDateBookMenu : public ListBasedDateBookMenu
{
public:
  DailyDateBookMenu(DateBook& dateBook,DateBookMenuCatalog* catalog);

  void mainLoop();

private:
  void showNext();
  void showPrevious();
};

// Derived class for string search view
class StringSearchDateBookMenu : public ListBasedDateBookMenu
{
public:
  StringSearchDateBookMenu(DateBook& dateBook, 
                           DateBookMenuCatalog* catalog);

  std::string searchString() const { return searchString_; }
  void searchString(const std::string& s);

  void mainLoop();

private:
  std::string searchString_;
};

// Class to collect all of the date book menus
class DateBookMenuCatalog
{
public:
  DateBookMenuCatalog(DateBook& dateBook);

  // Use compiler-generated destructor
  // ~DateBookMenuCatalog();

  DailyDateBookMenu* dailyMenu();
  WeeklyDateBookMenu* weeklyMenu();
  MonthlyDateBookMenu* monthlyMenu();
  StringSearchDateBookMenu* stringSearchMenu();

private:
  // Copying would be very destructive, so disable it.
  DateBookMenuCatalog(const DateBookMenuCatalog&);
  DateBookMenuCatalog& operator=(const DateBookMenuCatalog&);

  // This object "owns" a copy of each menu type
  std::auto_ptr<DailyDateBookMenu> dailyMenu_;
  std::auto_ptr<WeeklyDateBookMenu> weeklyMenu_;
  std::auto_ptr<MonthlyDateBookMenu> monthlyMenu_;
  std::auto_ptr<StringSearchDateBookMenu> stringSearchMenu_;
};

#endif // DateBookMenu_dot_h
